#!/usr/bin/env python 
#
# Copyright (c) Leipzig, Madrid 1999-2015 Gert Wollny
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

# this program is used to translate the XML files obtained by running a mia-* program 
# into a Unix man page. 

import sys
import time
import calendar
import string

python_version = sys.version_info.major
if python_version >= 3:
    from html.entities import name2codepoint
else:
    from htmlentitydefs  import name2codepoint

import re

sys.dont_write_bytecode = True

modules = {'miareadxml' : [0, '', 'none://miareadxml.py' ]
           }


from miareadxml import parse_file

def get_date_string():
    lt = time.localtime(time.time())
    return "{} {} {}".format (lt.tm_mday, calendar.month_name[lt.tm_mon], lt.tm_year)

#taken from http://effbot.org/zone/re-sub.htm#unescape-html
    
def unescape(text):
    def fixup(m):
        text = m.group(0)
        if text[:2] == "&#":
            # character reference
            try:
                if text[:3] == "&#x":
                    return unichr(int(text[3:-1], 16))
                else:
                    return unichr(int(text[2:-1]))
            except ValueError:
                pass
        else:
            # named entity
            try:
                text = unichr(name2codepoint[text[1:-1]])
            except KeyError:
                pass
        return text # leave as is
    return re.sub("&#?\w+;", fixup, text)

def escape_dash(text): 
    return re.sub(r'-', r'\-', text) 

def clean (text):
    text = unescape(text)
    return escape_dash(text) 

def write_man_file(descr):
    name = escape_dash(descr.name)
    print(".TH {} 1 \"v{}\"  \"USER COMMANDS\"".format(escape_dash(descr.name), descr.version))
    print(".SH NAME")
    print(name,) 
    print("\- {}".format (clean(descr.whatis)))
    print(".SH SYNOPSIS")
    print(".B {}".format (clean(descr.basic_usage)))
    print(".SH DESCRIPTION")
    print(".B {}".format (name))
    print(descr.description)
    print(".SH OPTIONS")
    for g in descr.option_groups:
        if len(g.name) > 0:
            print(".SS {}".format(g.name))

        print(".RS")
        for o in g.options:
            o.print_man()
        print(".RE")
    
    handlerkeys = sorted(descr.handlers.keys())
    for k in handlerkeys: 
        h = descr.handlers[k]
        print(".SH PLUGINS: {}".format (h.name))
        for p in h.plugins:
            print(".TP 10")
            print(".B {}".format(p.name))
            if len(p.params) > 0: 
                print("{}, supported parameters are: ".format (p.text))
                print(".P")
                for o in p.params:
                    print(".RS 14")
                    o.print_man()
                    print(".RE")
            else:
                print(p.text)
                print(".P")
                if not p.no_params_info: 
                    print(".RS 14")
                    print("(no parameters)")
                    print(".RE")
                    
            if p.suffixes is not None:
                print(".RS 14")
                print("Recognized file extensions: ", p.suffixes)
                print(".RE")
                print(" ")

            if p.supported_types is not None:
                print(".RS 14")
                print("Supported element types: ")
                print(".RS 2")
                print(p.supported_types)
                print(".RE")
                print(" ")
                print(".RE")


    if descr.Example.text is not None and len(descr.Example.text) > 0:
            print(".SH EXAMPLE")
            print(clean(descr.Example.text))
            for c in descr.Example.code:
                print(".HP")
                print("{} {}".format (name, clean(c.text)))

    print(".SH AUTHOR(s)")
    print(clean(descr.author))
    
    print(".SH COPYRIGHT")
    print("""This software is Copyright (c) 1999\(hy2015 Leipzig, Germany and Madrid, Spain.
It comes  with  ABSOLUTELY  NO WARRANTY  and  you  may redistribute it under the terms of the GNU 
GENERAL PUBLIC LICENSE Version 3 (or later). For more 
information run the program with the option '\-\-copyright'.""")


X=parse_file(sys.argv[1])
write_man_file(X)



#!/usr/bin/env python
#
# Copyright (c) Leipzig, Madrid 2004-2010
# BIT, ETSI Telecomunicacion, UPM
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#


# call the program like 
#    convert3d-to-slices.py -i somadata%04d.hdr -o outputdir

import os
import sys

from optparse import OptionParser
from string import atoi
from string import split
from subprocess import Popen
from subprocess import PIPE
from subprocess import call

parser = OptionParser()

parser.add_option("-f", "--filenamedescr", type="string", dest="file_descr", 
                  action="store", help="input file name pattern", default="data%04d.hdr")
parser.add_option("-o", "--outout_dir", type="string", dest="output", default=".",
                 action="store", help="output directory")

(options, args) = parser.parse_args()

sequence_slices=-1
start_image = 0
while sequence_slices < 0:
    name = options.file_descr % (start_image)
    if os.path.exists(name):
        cmd = 'mia-3dgetsize -i %s | sed -e "s/<[0-9]*,[0-9]*,\([0-9]*\)>/\\1/"' % (name)
        cs = Popen(cmd, shell=True, stdout=PIPE)
        retval = cs.communicate()
        token = split(retval[0])
        sequence_slices=atoi(token[0])


for i in range(sequence_slices):
    path = "%s/%d" % (options.output, i)
    if not os.path.exists(path):
        os.mkdir(path)
    

run=True

while run: 
    name = options.file_descr % (start_image)
    tmpname = "%s/data" % options.output
    if os.path.exists(name):
        cmd = "mia-3dimagefilter -i %s -o ubyte.v convert && mia-3dgetslice -i ubyte.v -o %s -t png" % (name,tmpname)
        print cmd
        call(cmd, shell=True)
        for i in range(sequence_slices): 
            outname = "%s/%d/data%04d.png" % (options.output, i, start_image)
            tmpname2 = "%s%04d.png" % (tmpname, i)
            os.rename(tmpname2, outname)
        start_image = start_image + 1
    else:
        run=False


        



#!/usr/bin/env python 
#
# Copyright (c) Leipzig, Madrid 1999-2015 Gert Wollny
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

# this program is used to translate the XML files obtained by running 
# a mia-* program with --help-xml into a nipype interface 

import sys
import time
import calendar
import string
import re
from os import path

python_version = sys.version_info.major
if python_version >= 3:
    from html.entities import name2codepoint
else:
    from htmlentitydefs  import name2codepoint

from argparse import ArgumentParser
from argparse import RawTextHelpFormatter

sys.dont_write_bytecode = True

modules = {'miareadxml' : [0, '', 'none://miareadxml.py' ]
           }


nipype_header = """from nipype.interfaces.base import (
        TraitedSpec,
        CommandLineInputSpec,
        CommandLine,
        File,
        traits
    )
import os"""


from miareadxml import parse_file

def get_date_string():
    lt = time.localtime(time.time())
    return "{} {} {}".format(lt.tm_mday, calendar.month_name[lt.tm_mon], lt.tm_year)

#taken from http://effbot.org/zone/re-sub.htm#unescape-html
    
def unescape(text):
    def fixup(m):
        text = m.group(0)
        if text[:2] == "&#":
            # character reference
            try:
                if text[:3] == "&#x":
                    return unichr(int(text[3:-1], 16))
                else:
                    return unichr(int(text[2:-1]))
            except ValueError:
                pass
        else:
            # named entity
            try:
                text = unichr(htmlentitydefs.name2codepoint[text[1:-1]])
            except KeyError:
                pass
        return text # leave as is
    return re.sub("&#?\w+;", fixup, text)

def dash_to_underscore(text): 
    return re.sub(r'-', r'_', text) 

def clean (text):
    text = unescape(text)
    return escape_dash(text) 


class  NipypeOutput: 
    def __init__(self, input_file, output_file):
        self.input_file = input_file
        self.output_file = output_file
        self.descr = parse_file(input_file)
        self.out=open(output_file, 'w')
        self.ParamTable = {
            "2dbounds" : lambda i : self.create_vint_param(i), 
            "3dbounds" : lambda i : self.create_vint_param(i), 
            "vshort":    lambda i : self.create_vint_param(i), 
            "vint":      lambda i : self.create_vint_param(i), 
            "vlong":     lambda i : self.create_vint_param(i), 
            "vushort":   lambda i : self.create_vint_param(i), 
            "vuint":     lambda i : self.create_vint_param(i), 
            "vulong":    lambda i : self.create_vint_param(i), 

            "2dfvector" :lambda i : self.create_vfloat_param(i), 
            "3dfvector" :lambda i : self.create_vfloat_param(i), 
            "vdouble"   :lambda i : self.create_vfloat_param(i), 
            "vfloat"    :lambda i : self.create_vfloat_param(i), 

            "bool"  : lambda i : self.create_Bool_param(i),
            
            "short" : lambda i : self.create_Integral_param(i), 
            "int"   : lambda i : self.create_Integral_param(i), 
            "long"  : lambda i : self.create_Integral_param(i),
            "ushort": lambda i : self.create_Integral_param(i), 
            "uint"  : lambda i : self.create_Integral_param(i), 
            "ulong" : lambda i : self.create_Integral_param(i),
            
            "int16_t" : lambda i : self.create_Integral_param(i), 
            "int32_t"   : lambda i : self.create_Integral_param(i), 
            "int64_t"  : lambda i : self.create_Integral_param(i),
            "uint16_t": lambda i : self.create_Integral_param(i), 
            "uint32_t"  : lambda i : self.create_Integral_param(i), 
            "uint64_t" : lambda i : self.create_Integral_param(i), 
            "float" : lambda i : self.create_Float_param(i), 
            "double": lambda i : self.create_Float_param(i), 

            "vstring": lambda i : self.create_input_VString_param(i), 

            "dict"  : lambda i : self.create_Dict_param(i), 
            "set"  : lambda i : self.create_Set_param(i), 
            "factory" : lambda i : self.create_Factory_param(i)
        }
        

    def create_trait_input_param_start(self, param, trait, enums=""):
        dash_removed_name = dash_to_underscore(param.long)
        # this should be implemented for all python keywords 
        if dash_removed_name == "lambda":
            dash_removed_name = dash_removed_name + "_"
        self.out.write ( "\t{} = traits.{}({} desc=\"\"\"{}\"\"\", ".format(dash_removed_name, 
                                                                  trait, enums, param.text)), 

    def create_param_tail(self, param):
        if param.required: 
            self.out.write (', mandatory = True '), 
        self.out.write( ')\n')

    def create_Bool_param(self, param):
        self.create_trait_input_param_start(param, 'Bool')
        self.out.write('argstr="--{}" '.format(param.long)), 
        self.create_param_tail(param)

    def  create_vint_param(self, param):
        self.create_trait_input_param_start(param, 'ListInt')
        self.out.write('argstr="--{} %s", sep=","'.format(param.long)), 
        self.create_param_tail(param)
        
    def create_vfloat_param(self, param):
        self.create_trait_input_param_start(param, 'ListFloat')
        self.out.write('argstr="--{} %s", sep=","'.format(param.long)), 
        self.create_param_tail(param)

    def create_input_VString_param(self, param):
        self.create_trait_input_param_start(param, 'ListString')
        self.out.write('argstr="--{} %s", sep=","'.format(param.long)), 
        self.create_param_tail(param)

    def create_Integral_param(self, param):
        self.create_trait_input_param_start(param, 'Int')
        self.out.write('argstr="--{} %d" '.format(param.long)), 
        self.create_param_tail(param)

    def create_Float_param(self, param):
        self.create_trait_input_param_start(param, 'Float')
        self.out.write('argstr="--{} %f" '.format(param.long)), 
        self.create_param_tail(param)

    def create_Dict_param(self, param):
        self.create_trait_input_param_start(param, 'Enum', param.get_names_as_string())
        self.out.write('argstr="--{} %s" '.format(param.long)), 
        self.create_param_tail(param)

    def create_Set_param(self, param):
        self.create_trait_input_param_start(param, 'Enum', param.get_names_as_string())
        self.out.write('argstr="--{} %s" '.format(param.long)), 
        self.create_param_tail(param)

    def create_Factory_param(self, param):
        self.create_trait_input_param_start(param, 'String')
        self.out.write('argstr="--{} %s" '.format(param.long)), 
        self.create_param_tail(param)

    def create_input_String_param(self, param):
        self.create_trait_input_param_start(param, 'String')
        self.out.write('argstr="--{} %s" '.format(param.long)), 
        self.create_param_tail(param)

    def create_input_File_param(self, param):
        self.create_trait_input_param_start(param, 'File')
        self.out.write('argstr="--{} %s", exists=True '.format(param.long)), 
        self.create_param_tail(param)

    def create_output_String_param(self, param):
        self.out.write ( '\t{} = traits.String(desc="{}", '.format(dash_to_underscore(param.long), param.text)), 
        self.out.write('argstr="--{} %s" '.format(param.long)), 
        self.create_param_tail(param)

    def create_output_File_param(self, param):
        self.out.write ( '\t{} = File(desc="{}", '.format(dash_to_underscore(param.long), param.text)), 
        self.out.write('argstr="--{} %s" '.format(param.long)), 
        self.create_param_tail(param)

    def create_output_File_param_outspec(self, param):
        self.out.write ( '\t{} = File(desc="{}"'.format(dash_to_underscore(param.long), param.text)), 
        if param.required: 
                self.out.write (', exists = True '), 
        self.out.write( ')\n')

    def write_unknown_type(self, param):
        print ("WARNING: Unknown  parameter type '{}' encounterd for option '{}', \n".format(param.long, param.type)), 
    
    def write_input_spec(self, name, inputs, params):

        self.out.write ("class {}_InputSpec(CommandLineInputSpec):\n".format(name))

        InputTable = self.ParamTable
        InputTable["string"] = lambda i :self.create_input_File_param(i)
        InputTable["io"] = lambda i :self.create_input_File_param(i)

        for i in inputs: 
            InputTable.get(i.type, self.write_unknown_type)(i)

        ParamTableCopy = self.ParamTable
        ParamTableCopy["string"] = lambda i :self.create_input_String_param(i)

        for i in params: 
            ParamTableCopy.get(i.type, self.write_unknown_type)(i)

    def write_input_outputs(self, name, outputs, params):

        InputTable = self.ParamTable
        InputTable["string"] = lambda i :self.create_output_File_param(i)
        InputTable["io"] = lambda i :self.create_output_File_param(i)

        for i in outputs: 
            InputTable.get(i.type, self.write_unknown_type)(i)

    def write_input_free_params_spec(self, freeparams):
        self.out.write ( '\tfree_params = traits.ListStr(desc="Plug-in specifications of type {}", '.format(freeparams)),
        self.out.write ( 'argstr="%s")')

    def write_output_spec(self, name, outputs, params):

        self.out.write ("class {}_OutputSpec(TraitedSpec):\n".format(name))

        ParamTableCopy = self.ParamTable
        ParamTableCopy["string"] = lambda i :self.create_output_File_param_outspec(i)
        ParamTableCopy["io"] = lambda i :self.create_output_File_param_outspec(i)

        for i in outputs: 
            ParamTableCopy.get(i.type, self.write_unknown_type)(i)

        if self.descr.stdout_is_result:
            self.out.write ( '\tstdout = traits.Str(\'result from stdout\')\n')
            
        self.out.write ("\n")

    def write_task(self, name, outputs):

        self.out.write('class {}(CommandLine):\n'.format(name))
        self.out.write('\tinput_spec = {}_InputSpec\n'.format(name))
        self.out.write('\toutput_spec = {}_OutputSpec\n'.format(name))
        self.out.write('\t_cmd = "{}"\n\n'.format(self.descr.name))

        self.out.write('\tdef _list_outputs(self):\n')
        self.out.write('\t\toutputs = self.output_spec().get()\n')

        for i in outputs: 
            # there is a problem here: for files that are given as pattern this will fail. 
            if i.type == "io" or i.type == "string":
                name = dash_to_underscore(i.long)
                self.out.write('\t\toutputs[\'{}\']=os.path.abspath(self.inputs.{})\n'.format(name,name))
            else:
                print('Only io or string supported for output, but got {}'.format(i))
        
        self.out.write('\t\treturn outputs\n')
       
        if self.descr.stdout_is_result:
            self.out.write('\tdef aggregate_outputs(self, runtime=None, needed_outputs=None):\n')
            self.out.write('\t\toutputs = super({}, self).aggregate_outputs(runtime, needed_outputs)\n'.format(name))
            self.out.write('\t\toutputs.stdout = runtime.stdout\n')
            self.out.write('\t\treturn outputs\n')

    def write_main(self, name):

        self.out.write( 'if __name__ == "__main__":\n')
        self.out.write( '\tmia_prog = {}()\n'.format(name))
        self.out.write( '\tprint( mia_prog.cmdline)\n')

    def write_nipype_file(self):

        name = dash_to_underscore(self.descr.name)

        inputs =[]
        outputs = []
        params = [] 

        for g in self.descr.option_groups:
            for o in g.options:
                if o.no_nipype:
                    continue 
                if o.is_input:
                    inputs.append(o)
                elif o.is_output:
                    outputs.append(o)
                else:
                    params.append(o)

        self.out.write("# This file was generated by running 'miaxml2nipype.py {} {}'\n".format(
                self.input_file, self.output_file))
        self.out.write("# '{}' can be created by running '{} --help-xml >{}'\n".format(
                self.output_file, self.descr.name, self.output_file))
        self.out.write("\n")

        self.out.write(nipype_header)
        self.out.write("\n#\n# Input specification\n#\n"); 
        self.write_input_spec(name, inputs, params)
        self.write_input_outputs(name, outputs, params)
        if self.descr.FreeParams is not None:
            self.write_input_free_params_spec(self.descr.FreeParams)

        self.out.write("\n#\n# Output specification\n#\n"); 
        self.write_output_spec(name, outputs, params)

        self.out.write("\n#\n# Task class specification\n#\n"); 
        self.write_task(name, outputs)

        self.out.write("\n#\n# Main function used for testing\n#\n"); 
        self.write_main(name)
   
  

def SanitizeName(matchobj):
    s = matchobj.group(0)
    if s[0] == '-':
        s = s[1:]
    return s.upper() 
         

#
# main program 
#

parser = ArgumentParser(description='Convert MIA style command line tools description into a nipype interface.', 
                                         formatter_class=RawTextHelpFormatter)
        
group = parser.add_argument_group('File I/O')
group.add_argument("-i", "--input",  action="store", required=True,  help="input XML file containing the description")
group.add_argument("-o", "--output", action="store", help="output file name, if not given, the name will be deducted from the input file")
group.add_argument("-p", "--outpath", action="store", help="output path for the nipype interface file")


options = parser.parse_args()

if options.output is None:
    (name , ext) = path.splitext(options.input)
    options.output = re.sub(r'(^[a-z]|-[a-z]|-[0-9][a-z])', SanitizeName, name) + ".py"

if options.outpath is not None:
    options.output = path.join(options.outpath, options.output)


a = NipypeOutput(options.input, options.output)
a.write_nipype_file()




# Copyright (c) Leipzig, Madrid 1999-2015 Gert Wollny
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

from lxml import etree
import re




modules = {'miareadxml' : [0, '', 'none://miareadxml.py' ]}

from miareadxml import get_text_node_simple
from miareadxml import get_dict_table
from miareadxml import make_sec_ancor
from miareadxml import xml_namespace
from miareadxml import xmlns

NSMAP={"xml" : xml_namespace }

def make_section_root_node(tag, name):
    secid = make_sec_ancor("Sec", name)
    section = etree.Element(tag)
    section.set(xmlns + "id", secid)
    section.set("xreflabel", name)
    title = etree.Element("title")
    title.text = name
    section.append(title)
    return section

def get_text_node(tag, renderas, text):
    node = etree.Element(tag)
    node.set("renderas", renderas)
    node.text = text
    return node


def get_bridgehead(text):
    return get_text_node("bridgehead", "sect3", text)
   
def get_synopsis(program):
    synopsis = etree.Element("cmdsynopsis")
    synopsis.append(get_text_node_simple("command", program.name))
    
    # add required options 
    for g in program.option_groups:
        for o in g.options:
            if o.required:
                node = etree.Element("arg", choice="req")
                node.text = "-" + o.short + " "
                node.append(get_text_node_simple("replaceable",  o.type))
                synopsis.append(node)
    
    # add standard option hint 
    node = etree.SubElement(synopsis, "arg", rep="repeat")
    node.append(get_text_node_simple("replaceable",  "options"))
    
    #add free parameter if exists
    if not program.FreeParams is None:
           node = etree.SubElement(synopsis, "arg", rep="repeat")
           plugin = etree.SubElement(node, "replaceable")
           etree.SubElement(plugin, "xref", linkend=make_sec_ancor("SecPlugintype", program.FreeParams))
    
    return synopsis

   
def get_option_descr(option):
    entry = etree.Element("varlistentry")

    option.write_xml(entry)
    
    return entry
                

def get_program(program):
    section = make_section_root_node("section", program.name)
    section.append(get_bridgehead("Sysnopis:"))
    section.append(get_synopsis(program))
    section.append(get_bridgehead("Description:"))
    section.append(get_text_node_simple("para", program.description))
    section.append(get_bridgehead("Options:"))
    
    for g in program.option_groups:
        if len(g.name) > 0:
            section.append(get_bridgehead(g.name))
            
        if len(g.options) > 0:
            varlist = etree.Element("variablelist", spacing="compact")
            for o in g.options:
                varlist.append(get_option_descr(o))
            section.append(varlist)

    if not program.Example is None:
        if not program.Example.text is None:
            section.append(get_bridgehead("Example:"))
            section.append(get_text_node_simple("para", program.Example.text))
            for c in program.Example.code:
                if not c.text is None: 
                    screen = etree.SubElement(section, "screen")
                    screen.text = program.name + " " + c.text
        else:
            print( "Warning: {} doesn't provide example text".format (program.name)  )
    else:
        print( "Warning: {} doesn't provide an example".format (program.name)  )

    section.append(get_bridgehead("Author(s):"))
    section.append(get_text_node_simple("para", program.author))
    
    return section 

def translate_descr(section, description):
    descrpara = etree.SubElement(section, "para", role="sectiondescr")
    for d in description:
        para = etree.SubElement(descrpara, "para", role="sectiondescr")
        para.text = d.text
        for l in d.iter("link"):
            link = etree.SubElement(para, "xref", linkend=make_sec_ancor(l.get("key"), l.text))
            link.tail = l.tail
    return descrpara

def get_section(name, sect):
    section = make_section_root_node("section", name)
    if sect.description is not None:
        descr = translate_descr(section, sect.description)
    else:
        print( "Warning: Section ", name, " doesn't have a description" )

    para = etree.SubElement(section, "para", role="sectiontoc")
        
    sect = sorted(sect.programs, key=lambda p: p.name )
    for program in sect:
        subpara = etree.SubElement(para, "para", role="sectiontoc")
        etree.SubElement(subpara, "xref", linkend=program.anchor)
        descr = etree.SubElement(subpara, "para", role="progdescr")
        descr.text = program.description
        section.append(get_program(program))
    return section


def get_plugin(plugin):
   result = etree.Element("para", role="plugin")
   plugin.write_xml(result)
   return result

def get_plugins(name, handler):
#   print( name )
   section = make_section_root_node("section", "Plugin type: " + name)
   para = etree.SubElement(section, "para", role="plugdescr")
   para.text = handler.description 
   head = get_bridgehead("Plugins:")
   section.append(head)
   plugshort = etree.SubElement(section, "para", role="pluginlist")
   
   for p in handler.plugins:
       etree.SubElement(plugshort, "xref", linkend=p.ancor, endterm=p.altancor )
       section.append(get_plugin(p))
      
   head = get_bridgehead("Plugin consumers:")
   section.append(head)
   para = etree.SubElement(section, "para", role="consumer")
   handler_users = []
   for u in handler.users:
       handler_users.append(u)
   handler_users = sorted(handler_users)
   for u in handler_users:
      etree.SubElement(para, "xref", linkend=u)

   return section



#
# Copyright (c) Leipzig, Madrid 1999-2015 Gert Wollny
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

from __future__ import print_function
import string
from lxml import etree
import re


xml_namespace = "http://docbook.org/ns/docbook"
xmlns = "{{}}".format(xml_namespace)

# supported tags: 
#   program      main tag 
#   name         text: program name
#   version      text: package version number 
#   section      text: program type
#   description  text: basic description of the program 
#   basic_usage  text: generic call 
#   group        tag name: option group name  
#     option     tag short: short option 
#                tag long:  long option 
#                tag required: 1 if required argument 
#                tag default: default option value
#                text: help 
#          flags
#                test; flags 
#       dict          
#  Example       text: example descripton 
#    Code        text: example text without program name 

def make_sec_ancor(key, text):
    """remove spaces and hyphens from the input string""" 
    return key + re.sub("[ -/:]", "", text)


def get_text_node_simple(tag,  text):
    node = etree.Element(tag)
    node.text = text
    return node


def escape_dash(text):
    return re.sub(r'-', r'\-', text)

def get_dict_table(dictionary, tabletype):
    entry = etree.Element(tabletype, frame="none")
    tgroup = etree.Element("tgroup", cols="2", colsep="0", rowsep ="0")
    colspec = etree.Element("colspec", colname="c1")
    tgroup.append(colspec)
    colspec = etree.Element("colspec", colname="c2")
    tgroup.append(colspec)

    tbody = etree.Element("tbody")
    for d in dictionary.keys(): 
        row = etree.Element("row")
        e = etree.Element("entry", align="left", valign="top")
        e.text = d + ":"
        row.append(e)
        e = etree.Element("entry", align="left", valign="top")
        e.text = dictionary[d]
        row.append(e)
        tbody.append(row)
    tgroup.append(tbody)
    entry.append(tgroup)
    return entry

class CTextNode: 
    def __init__(self, node, expect = None):
        if not expect is None and node.tag != expect:
            raise ValueError("expected '{}' got '{}'".format (expect, node.tag))

        self.required = False
        self.is_input = False
        self.is_output = False
        self.no_nipype = False
        self.entry = node.tag
        self.flags = set()
        self.text  = ""

        if node.text is not None:
            self.text = self.text + node.text

        for child in node.iter("flags"):
            f = child.text.split()
            for ff in f: 
                self.flags.add(ff) 
            if child.tail is not None:
                self.text = self.text + child.tail

        for f in self.flags: 
                p = {
                    "required": self.set_is_required, 
                    "input":    self.set_is_input, 
                    "output":   self.set_is_output,
                    "nonipype": self.set_nipype_ignore,
                    }.get(f, self.dummy)(f)
    
    def set_is_required(self, f):
        self.required = True

    def set_is_input(self, f):
        self.is_input = True

    def set_is_output(self, f):
        self.is_output = True

    def set_nipype_ignore(self, f):
        self.no_nipype = True

    def dummy(self, f):
        warnings.warn ('Unknown flag "{}" encountered'.format(f))


class COption(CTextNode):
    def __init__(self, node):
        CTextNode.__init__(self, node, "option")

        self.short = node.get("short")
        self.long =  node.get("long")
#        self.required = int(node.get("required")) 
        self.default = node.get("default")
        self.type = node.get("type")
        self.value_range = None
       
        for r in node:
            if r.tag == "range":
                self.value_range  = "[" + r.get("min") + ', ' + r.get("max") + "]"
            if r.tag == "bounded":
                self.value_range  = r.get("min") + ', ' + r.get("max")


    def print_man(self):
        if len(self.short) > 0:
            short = "\-" + self.short
        else:
            short = "  ";

        if len(self.flags) > 0:
            flagstring = ""
            for f in self.flags:
                if f != "nonipype":
                    if len(flagstring) == 0:
                        flagstring = f
                    else:
                        flagstring = flagstring + ', ' + f
            if self.value_range is not None: 
                print(".IP \"{} \-\-{}=({}); {} in {}\"".format (short, self.long, flagstring, self.type, self.value_range))
            else:
                if not self.type == "bool":
                    print( ".IP \"{} \-\-{}=({}); {}\"".format (short, self.long, flagstring, self.type))
                else:
                    print( ".IP \"{} \-\-{}\"".format (short, self.long))
        else:
            if not self.type == "bool":
                if self.value_range is not None: 
                    print( ".IP \"{} \-\-{}={}; {} in {}\"".format (short, self.long, escape_dash(self.default), self.type, self.value_range))
                else:
                    print( ".IP \"{} \-\-{}={}\"".format (short, self.long, escape_dash(self.default)))
            else:
                print( ".IP \"{} \-\-{}\"".format (short, self.long))
        print( escape_dash(self.text), )
        self.do_print_man()


    def do_print_man(self):
            print( "" )

    def write_xml(self, entry):
        # create the terminal text 
        termtext = "-"
        if len(self.short) > 0: 
            termtext = termtext + self.short + ", -"
        termtext = termtext + "-" + self.long

        if len(self.flags) > 0:
            termtext = termtext + "=("
            first = True
            for f in self.flags:
                if f != "nonipype":
                    if first: 
                        termtext = termtext + f
                        first=False
                    else:
                        termtext = termtext + ", " + f
            if self.type is not None: 
                termtext = termtext + "); " + self.type
            if self.value_range is not None: 
                termtext = termtext + " in "  + self.value_range
        elif self.type != "bool":
            termtext = termtext + "="
            if len(self.default)>0:
                termtext = termtext + self.default + "; " + self.type
            if self.value_range is not None: 
                termtext = termtext + " in "  + self.value_range
            elif self.type == "string":
                termtext = termtext + "STRING"

        entry.append(get_text_node_simple("term",  termtext))
        item =  etree.Element("listitem")
        para = get_text_node_simple("para", self.text)
        self.do_write_xml(para)
        item.append(para)
        entry.append(item)

    def do_write_xml(self, parent):
        pass 


class CDictOption(COption):
    def __init__(self, node):
        COption.__init__(self, node)
        self.dict = {}
        for child in node.iter("dict"):
            for v in child:
                self.dict[v.get("name")] = v.text
    def get_names_as_string(self):
        result = ""
        for k in self.dict.keys():
            result = result + '"' + k + '", '
        return result

    def do_print_man(self):
        if len(self.dict) > 0:
            print( "" )
            for k in self.dict.keys(): 
                print( ".RS 10" )
                print( ".I" )
                print( escape_dash(k) )
                print( "\(hy {}".format (self.dict[k]) )
                print( ".RE" )


    def do_write_xml(self, parent):
        if len(self.dict) > 0:
            opttable = get_dict_table(self.dict, "informaltable")
            parent.append(opttable)


class CSetOption(COption):
    def __init__(self, node):
        COption.__init__(self, node)
        self.set = []
        for child in node.iter("set"):
            for cc in child.iter("value"):
                self.set.append(cc.get("name"))
                
    def get_names_as_string(self):
        result = ""
        for k in self.set:
            result = result + '"' + k + '", '
        return result

    def do_print_man(self):
        if len(self.set) > 0:
            print( "" )
            print( ".RS 10" )
            print( ".I" )

            print( "Supported values are:(", )
            for k in self.set:
                print( "{}, ".format (escape_dash(k)), )
            print( ")" )
            print( ".RE" )


    def do_write_xml(self, parent):
        if len(self.set) > 0:
            e = etree.SubElement(parent, "entry", align="left", valign="top")
            str_list = [" Supported values are:("]
            
            for k in self.set:
                str_list.append("{}, ".format (k))
            str_list.append(")")
            e.text = ''.join(str_list)
                

class CIOOption(COption):
    def __init__(self, node):
        COption.__init__(self, node)
        self.factory = "unknown/io"
        for child in node.iter("io"):
            self.factory = child.get("name")
            if child.tail is not None:
                if self.text is not None: 
                    self.text = self.text + child.tail
                else:
                    self.text = child.tail

    def do_print_man(self):
        print( " For supported file types see PLUGINS:{}".format (self.factory) )

    def do_write_xml(self, parent):
        parent.text = parent.text + ". For supported file types see "
        etree.SubElement(parent, "xref", linkend=make_sec_ancor("SecPlugintype", self.factory))
        
class CFactoryOption(COption):
    def __init__(self, node):
        COption.__init__(self, node)
        self.factory = "unknown/factory"
        for child in node.iter("factory"):
            self.factory = child.get("name")
            if child.tail is not None:
                if self.text is not None: 
                    self.text = self.text + child.tail
                else:
                    self.text = child.tail

    def do_print_man(self):
        print( " For supported plugins see PLUGINS:{}".format (self.factory))

    def do_write_xml(self, parent):
        parent.text = parent.text + ". For supported plug-ins see "
        etree.SubElement(parent, "xref", linkend=make_sec_ancor("SecPlugintype", self.factory))


class CExample(CTextNode):
    def __init__(self, node):
        CTextNode.__init__(self, node, "Example")
        self.code = []

        for child in node:
            if child.tag == "Code": 
                self.code.append(CTextNode(child, "Code"))
            else:
                print( "unexpected subnode '{}' in example".format (child.tag) )


class CGroup:
    def __init__(self, node):
        if node.tag != "group":
            raise ValueError("expected 'group' got '{}'".format (node.tag))

        self.entry = node.tag
        self.name  = node.get("name")

        self.options = []
        for child in node:
            if child.tag == "option": 
                p = {
                    "io": lambda n: CIOOption(n), 
                    "factory": lambda n: CFactoryOption(n), 
                    "dict":    lambda n: CDictOption(n),
                    "set": lambda n: CSetOption(n),
                    }.get(child.get("type"), lambda n: COption(n))(child)
                self.options.append(p)
            else:
                print( "unexpected subnode '{}' in 'group'".format (child.tag) )

def get_text_element(root, name):
    xname = etree.SubElement(root, name)
    if xname is None:
        raise ValueError("Program description misses the tag {}".format (name))
    return xname.text

class CParam: 
    def __init__(self, node):
        if node.tag != "param":
            raise ValueError("expected 'param' got '{}'".format (node.tag))
        self.name = node.get("name")
        self.type = node.get("type")
        self.default = node.get("default")
        self.text = node.text
        self.required = False
        self.no_nipype = False 
        self.flags = set()

        for child in node.iter("flags"):
            f = child.text.split()
            for ff in f: 
                self.flags.add(ff) 
            if child.tail is not None:
                self.text = self.text + child.tail

        m = re.search('[,=:]', self.default) 
            # if there is a ',' in the text make clean that it needs to be escaped  
        if m is not None: 
            self.default = "[" + self.default + "]"

        for f in self.flags: 
                p = {
                    "required": self.set_is_required, 
                    "input":    self.set_is_input, 
                    "output":   self.set_is_output,
                    "nonipype": self.set_nipype_ignore,
                    }.get(f, self.dummy)(f)
    
    def set_is_required(self, f):
        self.required = True

    def set_is_input(self, f):
        self.is_input = True

    def set_is_output(self, f):
        self.is_output = True
        
    def set_nipype_ignore(self, f):
        self.no_nipype = True

    def dummy(self, f):
        warnings.warn ('Unknown flag "{}" encountered'.format(f))


    def print_man(self):
        print( ".I" )
        print( self.name )


        if len(self.flags) > 0:
            termtext = ""
            for f in self.flags:
                if f == "nonipype":
                    continue 
                if len(termtext) == 0:
                    termtext = "=(" + f
                else:
                    termtext = termtext + ', ' + f

            termtext = termtext + ", {})"
            print( termtext.format(self.type))
        elif self.required:
            print( "= (required); {} ".format (self.type))
        else:
            print( "= {}; {} ".format (escape_dash(self.default), self.type))
        print( ".RS 2" )
        print( "{}.".format (escape_dash(self.text)) )
        self.do_print_man()
        print( ".RE" )

    def do_print_man(self):
        print( "" )

    def print_xml_help(self, root):
        row = etree.SubElement(root, "row")
        e = etree.SubElement(row, "entry", align="center", valign="top")
        e.text = self.name 
        e = etree.SubElement(row, "entry", align="center", valign="top")
        e.text = self.type
        e = etree.SubElement(row, "entry", align="center", valign="top")

        if len(self.flags) > 0:
            first = True
            for f in self.flags:
                if f == "nonipype": 
                    continue
                if first:
                    e.text = "(" + f
                    first = False
                else:
                    e.text = e.text + ',' + f
            e.text = e.text + ')'
        elif self.required: 
            e.text = "(required)"
        else:
            e.text = self.default

        self.do_print_xml_help_description(row)

    def do_print_xml_help_description(self, row):
        e = etree.SubElement(row, "entry", align="left", valign="top")
        e.text = self.text

    def append_to_handler(self, handlers, link):
        pass

class CRangeParam(CParam):
    def __init__(self, node):
        CParam.__init__(self,node)
        self.value_range = None
        
        for r in node:
            if r.tag == "range":
                self.type = self.type + " in [" + r.get("min") + ", " + r.get("max") + "]"
            if r.tag == "bounded":
                self.type = self.type + " in " + r.get("min") + ", " + r.get("max")
                

    def do_print_xml_help_description(self, row):
        e = etree.SubElement(row, "entry", align="left", valign="top")
        e.text = self.text



class CDictParam(CParam):
    def __init__(self, node):
        CParam.__init__(self,node)
        self.dict = {}
        for n in node:
            if n.tag == "dict":
                for v in n:
                    self.dict[v.get("name")] = v.text

    def do_print_man(self):
        print( "Supported values are:" )
        for k in self.dict.keys(): 
            print( ".RS 4" )
            print( ".I" )
            print( escape_dash(k) )
            print( "\(hy {}".format (self.dict[k]) )
            print( ".RE" )
        CParam.do_print_man(self)

    def do_print_xml_help_description(self, row):
        e = etree.SubElement(row, "entry")
        table = etree.SubElement(e, "informaltable", pgwide="1") 
        tgroup = etree.SubElement(table, "tgroup", cols="2")
        colspec = etree.SubElement(tgroup, "colspec", colname="sc0")
        colspec = etree.SubElement(tgroup, "colspec", colname="sc1")
        tbody = etree.SubElement(tgroup, "tbody")

        subrow = etree.SubElement(tbody, "row")
        e = etree.SubElement(subrow, "entry", align="left", valign="top", namest="sc0", nameend="sc1")
        e.text = self.text 
        for d in self.dict.keys(): 
            subrow = etree.SubElement(tbody, "row")
            e = etree.SubElement(subrow, "entry", align="left", valign="top")
            e.text = d + ":"
            e = etree.SubElement(subrow, "entry", align="left", valign="top")
            e.text = self.dict[d]
        return e

class CSetParam(CParam):
    def __init__(self, node):
        CParam.__init__(self,node)
        self.set = []
        for n in node:
            if n.tag == "set":
                for v in n:
                    if v.tag == "value":
                        self.set.append(v.get("name"))

    def do_print_man(self):
        print( "Supported values are:(",  )
        for k in self.set:
            print( "{}, ".format (escape_dash(k)),  )
        print( ")" )
        CParam.do_print_man(self)

    def do_print_xml_help_description(self, row):
        e = etree.SubElement(row, "entry", align="left", valign="top")
        str_list = [self.text, " Supported values are:("]

        for k in self.set:
            str_list.append("{}, ".format (k))
        str_list.append(")")
        e.text = ''.join(str_list)

class CFactoryParam(CParam):
    def __init__(self, node):
        CParam.__init__(self,node)
        self.factory = ""
        for n in node:
            if n.tag == "factory":
                self.factory = n.get("name")

    def do_print_man(self):
        print( "For supported plug-ins see PLUGINS:{}".format (self.factory) )
        CParam.do_print_man(self)

    def do_print_xml_help_description(self, row):
        e = etree.SubElement(row, "entry", align="left", valign="top")
        e.text = self.text + ". For supported plug-ins see "
        etree.SubElement(e, "xref", linkend=make_sec_ancor("SecPlugintype", self.factory))


    def append_to_handler(self, handlers, link):
        if self.factory in handlers:
            handlers[self.factory].append_user(link)
        else:
            raise RuntimeError("Handler {} is used by plugin {}, but is not available".format (self.factory, link))


class CIOParam(CParam):
    def __init__(self, node):
        CParam.__init__(self,node)
        self.factory = ""
        for n in node:
            if n.tag == "io":
                self.factory = n.get("name")

    def do_print_man(self):
        print( "For supported file types see PLUGINS:{}".format (self.factory) )
        CParam.do_print_man(self)

    def do_print_xml_help_description(self, row):
        e = etree.SubElement(row, "entry", align="left", valign="top")
        e.text = self.text + ". For supported file types see "
        etree.SubElement(e, "xref", linkend=make_sec_ancor("SecPlugintype", self.factory))


    def append_to_handler(self, handlers, link):
        if self.factory in handlers:
            handlers[self.factory].append_user(link)
        else:
            raise RuntimeError("Handler {} is used by plugin {}, but is not available".format (self.factory, link))

class CPlugin: 
    def __init__(self, node, handlername):
        if node.tag != "plugin":
            raise ValueError("expected 'plugin' got '{}'".format (node.tag))
        self.name = node.get("name")
        self.text = node.text
        self.handlername = handlername
        self.ancor = make_sec_ancor("plugin"+self.name, self.handlername)
        self.altancor = self.ancor + "alt"
        self.no_params_info = False
        self.params = []
        self.supported_types = None
        self.suffixes = None
        for child in node:
            if child.tag == "param": 
                p = {
                    "int":   lambda n: CRangeParam(n),
                    "uint":   lambda n: CRangeParam(n),
                    "float":   lambda n: CRangeParam(n),
                    "double":   lambda n: CRangeParam(n), 
                    "factory": lambda n: CFactoryParam(n),
                    "io":      lambda n: CIOParam(n), 
                    "set":     lambda n: CSetParam(n),
                    "dict":    lambda n: CDictParam(n),
                    }.get(child.get("type"), lambda n: CParam(n))(child)

                self.params.append(p)
            elif child.tag == "noparam": 
                self.no_params_info = True
            elif child.tag == "datatypes":
                self.supported_types = child.text
            elif child.tag == "suffixes":
                self.suffixes = child.text
            else:
                print( "unexpected subnode '{}' in 'plugin'".format (child.tag) )

    def append_to_handler(self, handlers):
        for p in self.params:
            p.append_to_handler(handlers, self.ancor)


    def write_xml(self, page):
        parent = etree.SubElement(page, "sect4",  xreflabel=self.name+":"+self.handlername)
        parent.set(xmlns+"id", self.ancor)
        
        head = etree.SubElement(parent, "title",)
        head.text = self.name        
        
        altancor = etree.SubElement(parent, "titleabbrev")
        altancor.set(xmlns+"id", self.altancor)
        altancor.text = self.name 

        node = etree.SubElement(parent, "para", role="plugindescr")
        param_list = self.params

        if not self.no_params_info: 
            if len(param_list) > 0:
                node.text = self.text + ". Supported parameters are:"
                table = etree.SubElement(node, "informaltable", frame="all", role="pluginparms", pgwide="1")
                tgroup = etree.SubElement(table, "tgroup", cols="3", colsep="0", rowsep ="0")
                colspec = etree.SubElement(tgroup, "colspec", colname="c1", colwidth="10%")
                colspec = etree.SubElement(tgroup, "colspec", colname="c2", colwidth="20%")
                colspec = etree.SubElement(tgroup, "colspec", colname="c3", colwidth="10%")
                colspec = etree.SubElement(tgroup, "colspec", colname="c4", colwidth="60%")
                thead = etree.SubElement(tgroup, "thead")
                row = etree.SubElement(thead, "row"); 
                e = etree.SubElement(row, "entry", align="center", valign="top")
                e.text = "Name"
                e = etree.SubElement(row, "entry", align="center", valign="top")
                e.text = "Type"
                e = etree.SubElement(row, "entry", align="center", valign="top")
                e.text = "Default"
                e = etree.SubElement(row, "entry", align="center", valign="top")
                e.text = "Description"
                tbody = etree.SubElement(tgroup, "tbody")
                
                for p in param_list: 
                    p.print_xml_help(tbody)
            else:
                node.text = self.text + ". (This plug-in doesn't take parameters)"
        else:
            node.text = self.text

        if self.suffixes is not None:
            suffixes = etree.SubElement(node, "para", role="pluginsubdescr")
            suf = etree.SubElement(suffixes, "emphasis")
            suf.text = "Recognized file extensions: "
            suf.tail =  self.suffixes

        if self.supported_types is not None:
            datatypes = etree.SubElement(node, "para", role="pluginsubdescr")
            data = etree.SubElement(datatypes, "emphasis")
            data.text = "Supported element types: "
            data.tail = self.supported_types

class CHandler: 
    def __init__(self, node):
        if node.tag != "handler":
            raise ValueError("expected 'handler' got '{}'".format (node.tag))
        self.entry = node.tag
        self.name =  node.get("name")
        self.users = set([])
        self.description = node.text 
        self.plugins = []

        for child in node:
            if child.tag == "plugin": 
                self.plugins.append(CPlugin(child, self.name))
            else:
                print( "unexpected subnode '{}' in 'handler'".format (child.tag) )

    def append_user(self, user):
        self.users.add(user)

    def merge_users(self, users):
        self.users = self.users.union(users)




class CDescription: 
    def __init__(self, node):
        self.Example = None 
        self.FreeParams = None
        self.stdout_is_result = False
        self.option_groups = []
        self.handlers = {}
        for n in node:
            if n.tag == 'name':
                self.name = n.text
            elif n.tag == 'section':
                self.section = n.text
            elif n.tag == 'version':
                self.version = n.text
            elif n.tag == 'whatis':
                self.whatis = n.text
            elif n.tag == 'description':
                self.description = n.text
            elif n.tag == 'basic_usage':
                self.basic_usage = n.text
            elif n.tag == 'handler':
                handler = CHandler(n)
                self.handlers[handler.name] = handler
            elif n.tag == 'group': 
                self.option_groups.append(CGroup(n))
            elif n.tag == 'Example': 
                self.Example = CExample(n)
            elif n.tag == 'Author': 
                self.author = n.text
            elif n.tag == 'freeparams':
                self.FreeParams = n.get("name")
            elif n.tag == "stdout-is-result":
                self.stdout_is_result = True
            else: 
                print( "unknown tag '{}'".format (n.tag) )
        self.anchor = make_sec_ancor("Sec", self.name)
        self.link_handler_consumers()

    def link_handler_consumers(self):
        for h in self.handlers.keys():
            self.handlers[h].append_user(self.anchor) 
            for p in self.handlers[h].plugins:
                p.append_to_handler(self.handlers)
        if not  self.FreeParams is None:
            handler = self.handlers[h] 
            handler.append_user(self.anchor)

def parse_file(xmlpath):
    file=open(xmlpath, "r")
    stringstree = file.read()
    root = etree.XML(stringstree)
    if root.tag != "program": 
        raise ValueError("Expected tag 'program' not found. '"+xmlpath+"' is not a mia program descripion.")

    return CDescription(root)



#!/usr/bin/env python 
#
# Copyright (c) Leipzig, Madrid 1999-2015 Gert Wollny
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

#
# This program is used to read the mia-*.xml files that result from running 
# the mia-* programs with the --help-xml option and convert the information 
# into two files program.xml and plugins.xml that are included into the userref.xml 
# file which forms a Docbook 5.0 document then ready to be feeded to xslproc 
# for the creation of the HTML reference. 
#


import sys
import time
import calendar
import string
from lxml import etree
import re
from os import listdir
from fnmatch import fnmatch

sys.dont_write_bytecode = True

modules = {'miareadxml' : [0, '', 'none://miareadxml.py' ],
           'miawritprogram' : [0, '', 'none://miawritprogram.py' ]
           }


from miareadxml import parse_file
from miawritprogram import make_sec_ancor
from miawritprogram import get_section
from miawritprogram import get_plugins
from miawritprogram import xml_namespace
from miawritprogram import xmlns
from miawritprogram import NSMAP


def create_text_node(tag, text):
    node = etree.Element(tag)
    node.text = text
    return node


def get_date_string():
    lt = time.localtime(time.time())
    return "{} {} {]".format(lt.tm_mday, calendar.month_name[lt.tm_mon], lt.tm_year)

def find_files(root, pattern): 
    files = listdir(root)
    outfiles = []
    for f in files:
        if fnmatch(f, pattern):
            outfiles.append(f)

    return outfiles 

class CProgramSection:
    def __init__(self, node=None, name=None, idx=0):
        self.programs=[]
        self.sortindex = idx
        if node is not None: 
            self.name=node.get("name")
            self.description = node.iter("description")
        else:
            self.name=name
            self.description = None

def read_section_file(filename):
    file=open(filename, "r")
    stringstree = file.read()
    root = etree.XML(stringstree)

    program_sections = {}

    if root.tag != "outline": 
        raise ValueError("Expected tag 'outline' not found. sections.xml is not the expected file.")
    index = 0
    for n in root:
        section = CProgramSection(node=n, idx=index)
        program_sections[section.name] = section
        index = index+1
    return program_sections
    

rootpath = sys.argv[1]
outpath = sys.argv[2]
#
# read all xml files created by the command line tools 
#
files = find_files(outpath, "mia-*.xml")
descriptions=[]

for f in files:
    try:
        d = parse_file(f)
        descriptions.append(d)
    except ValueError as e:
        print (e)


#
# Read the program section description, this also provides the general structure  
# of the program reference
#
program_sections = read_section_file(rootpath + "/sections.xml")
index = len(program_sections)

#
# Sort the programs into the sections and read the plug-in types 
#
plugin_types = {}
for d in descriptions:
   if not d.section in program_sections:
      program_sections[d.section] = CProgramSection(name=d.section, idx=index)
      index = index + 1
      print( "Warning: Program '{}' is listed in section '{}', but this section is not described in doc/sections.xml".format(d.name, d.section))
   program_sections[d.section].programs.append(d)

   for h in d.handlers.keys():
      k = d.handlers[h].name
      if not k in plugin_types:
         plugin_types[k] = d.handlers[h]
      else:
         plugin_types[k].merge_users(d.handlers[h].users)



#
# Now convert to the docbook 5.0 format - these are also XML files 
# The output can then be parsed and converted by xsltproc 
#
prog_xml = etree.Element("chapter", nsmap=NSMAP)
prog_xml.set(xmlns + "id", "Programs")
title = create_text_node("title", "Program Reference")
prog_xml.append(title)


sorted_sections = {}

for s in program_sections.keys():
    sorted_sections[program_sections[s].sortindex] = s

sorted_sections_keys = sorted(sorted_sections.keys())

for i in sorted_sections_keys:
    s = sorted_sections[i]
    prog_xml.append(get_section(s, program_sections[s]))

programs_xml = etree.tostring(prog_xml, pretty_print=True)
prog_file = open(outpath + "/program.xml", "wb")
prog_file.write(programs_xml)
prog_file.close()

plug_xml = etree.Element("chapter", id="plugins", nsmap=NSMAP)
title = create_text_node("title", "Plugin Reference")
plug_xml.append(title)

plugin_types_keys = sorted(plugin_types.keys())
for s in plugin_types_keys: 
   plug_xml.append(get_plugins(s, plugin_types[s]))

plugins_xml = etree.tostring(plug_xml, pretty_print=True)
plug_file = open(outpath + "/plugins.xml", "wb")
plug_file.write(plugins_xml)
plug_file.close()



